<?php defined('BASEPATH') OR exit('No direct script access allowed');

class Send extends CI_Controller
{
	public $data;
    private $ip;

	public function __construct()
    {
        parent::__construct();

        /** 检查是否已登录，未登录执行登录步骤 */
        $this->userm->check_login(2, FALSE);
        $this->data['UserInfo'] = $this->userm->UserInfo;
        $this->ip               = $this->input->ip_address();
        $this->load->helper('string');
    }

    /**
     * 发送验证码接口
     */
    public function code()
    {
	    $action = $this->uri->segment(3, '');
	    $code_type = $this->uri->segment(4, '');

	    /** @var 判断访问参数是否合法 $actions */
        $actions = [0 => 'auto', 1 => 'phone', 2 => 'email'];
        ( in_array( $action, $actions ) &&
            in_array( $code_type, [ 'verify', 'register', 'bindphone', 'updatephone', 'bindemail', 'updateemail', 'bindencashment', 'resetpwd' ] )
        ) OR ShowMsg( '非法操作', '-1' );

        /** 获取验证码自动判断是手机或邮箱 */
        $this->load->helper('email');
        if ($action == 'auto')
        {
            $account = $this->input->post_get('account', TRUE);
            if ( is_phone( $account ) )
            {
                $action       = 'phone';
                $account_name = '手机';
            }
            elseif ( valid_email( $account ) )
            {
                $action       = 'email';
                $account_name = '邮箱';
            }
            else
            {
                ShowMsg('请输入正确的手机号/邮箱', '-1');
            }
        }

        /** 固定手机或邮箱方式 */
        else
        {
            /** 旧手机/邮箱验证 */
            if(intval($this->input->post_get('oldverify', TRUE)))
            {
                $account = $this->data['UserInfo'][$action];
            }
            else
            {
                $account = $this->input->post_get($action, TRUE);
            }

            if ($action == 'phone')
            {
                is_phone($account) OR ShowMsg('请输入正确的手机号', '-1');
                $account_name = '手机';
            }
            elseif($action == 'email')
            {
                valid_email($account) OR ShowMsg('请输入正确的邮箱', '-1');
                $account_name = '邮箱';
            }
        }
        $code_type_id = array_search($action, $actions);

        /** 已登录，不用图片验证码 */
        if ( ! $this->data['UserInfo'])
        {
            /** @var 图片验证码 $verify_code */
            $this->load->helper('captcha');
            $verify_code = $this->input->post_get('verify_code', TRUE);
            //check_captcha($verify_code) OR ShowMsg('图片验证码错误', '-1');
        }

        /** 须先判断手机或邮箱是否存在的验证码发送 */
        if ( in_array( $code_type, [ 'register', 'resetpwd', 'bindphone', 'bindemail' ] ) )
        {
            $old = $this->userm->get([
                'where' => [$action => $account],
                'field' => 'user_id',
            ]);

            /** 注册 */
            if ( $code_type == 'register' and $old )
            {
                ShowMsg("{$account_name}已经被他人注册，请更换一个吧");
            }

            /** 找回密码 */
            elseif($code_type == 'resetpwd' && ! $old)
            {
                ShowMsg("此{$account_name}不存在，请更换已绑定的{$account_name}");
            }

            /** 绑定安全手机或安全邮箱 */
            elseif ( ( $code_type == 'bindphone' || $code_type == 'bindemail' ) && $old )
            {
                if ($old['user_id'] == $this->data['UserInfo']['user_id'])
                {
                    ShowMsg("不能绑定原来的{$account_name}，请更换一个吧");
                }
                else
                {
                    ShowMsg("此{$account_name}已经被他人绑定，请更换一个吧");
                }
            }
        }

        /** 每台电脑每天获取次数 */
        $this->load->model('verifycodem');
        $ip_count = $this->verifycodem->count([
            'where' => [
                'verify_type'   => $code_type_id,
                'add_ip'        => $this->ip,
                'FROM_UNIXTIME(`add_time`, \'%Y-%m-%d\') = curdate()' => NULL,
            ],
        ]);
        $ip_count < 10 OR ShowMsg('今天获取验证码次数已超出限制，请明天再试', '-1');

        /** 每个手机号或邮箱每天只能获取5次验证码 */
        $code_count = $this->verifycodem->count([
            'where' => [
                'phone_email' => $account,
                'verify_type' => $code_type_id,
                'FROM_UNIXTIME(`add_time`, \'%Y-%m-%d\') = curdate()' => NULL,
            ],
        ]);
        $code_count < 5 OR ShowMsg("此{$account_name}获取验证码次数已超出限制，请明天再试", '-1');
    
        /** 获取两分钟内生成并未使用的验证码 */
        $time     = now();
        $old_code = [];
        $result   = TRUE;
        if($code_count)
        {
            $old_code = $this->verifycodem->get([
                'where' => [
                    'phone_email' => $account,
                    'verify_type' => $code_type_id,
                    'verify_place'=> $this->verifycodem->verify_place[$code_type],
                    'status'      => 1,
                    'add_time > ' => ( $time - 120 ),
                ],
                'order_by' => 'add_time DESC',
            ]);
        }

        /** 生成验证码 */
        if( ! $old_code )
        {
            $sms_verify_code = random_string('numeric', 6);

            /** 发送验证码 */
            if ($action == 'phone')
            {
                $this->load->library('Smsbao/sms', NULL, 'smsbao_sms');
                    $result = $this->smsbao_sms->send($account, [
                    'code'    => $sms_verify_code
                ], $code_type)['status'];
            }
            elseif($action == 'email')
            {
                $result = send_email($account, $code_type, [
                    'code'      => $sms_verify_code,
                    'web_name'  => $this->config->item( 'web_name' ),
                ]);
            }

            /** 插入数据库 */
            if ($result === TRUE)
            {
                $result = (bool) $this->verifycodem->save([
                    'phone_email'   => $account,
                    'verify_type'   => $code_type_id,
                    'verify_place'  => $this->verifycodem->verify_place[$code_type],
                    'verify_code'   => $sms_verify_code,
                    'add_time'      => $time,
                    'expired_time'  => ( $time + 600 ),
                    'add_ip'        => $this->ip,
                    'add_client'    => USE_CLIENT,
                ]);
            }
        }

        if($result === TRUE) ShowMsg('发送成功', '-1', 1);
        else ShowMsg('发送失败', '-1');
	}
}
